@@ -3114,7 +3114,7 @@ if (!strncmp(name, "GL_", 3)) {
  */
 #if defined(GL_ALL_CLIENT_ATTRIB_BITS) && !defined(GL_CLIENT_ALL_ATTRIB_BITS)
 #define GL_CLIENT_ALL_ATTRIB_BITS GL_ALL_CLIENT_ATTRIB_BITS
-#else 
+#elif !defined(GL_ALL_CLIENT_ATTRIB_BITS) && defined(GL_CLIENT_ALL_ATTRIB_BITS)
 #define GL_ALL_CLIENT_ATTRIB_BITS GL_CLIENT_ALL_ATTRIB_BITS
 #endif
 	i(GL_ALL_CLIENT_ATTRIB_BITS)
