@@ -139,6 +139,8 @@
 	i(GLUT_NORMAL_DAMAGED)
 	i(GLUT_OVERLAY_DAMAGED)
 #endif
+		/* OS/2 PM implementation does not have these constants... */
+#if !defined(GLUT_MIDDLE_BUTTON) || defined(GLUT_NORMAL)
 	i(GLUT_NORMAL)
 	i(GLUT_OVERLAY)
 	i(GLUT_ACTIVE_SHIFT)
@@ -167,6 +169,21 @@
 	i(GLUT_CURSOR_INHERIT)
 	i(GLUT_CURSOR_NONE)
 	i(GLUT_CURSOR_FULL_CROSSHAIR)
+#endif
+#if GLUT_API_VERSION >= 4
+	i(GLUT_GAME_MODE_ACTIVE)
+	i(GLUT_GAME_MODE_POSSIBLE)
+	i(GLUT_GAME_MODE_WIDTH)
+	i(GLUT_GAME_MODE_HEIGHT)
+	i(GLUT_GAME_MODE_PIXEL_DEPTH)
+	i(GLUT_GAME_MODE_REFRESH_RATE)
+	i(GLUT_GAME_MODE_DISPLAY_CHANGED)
+#endif
+	/* FreeGLUT Constants */
+	i(GLUT_ACTION_EXIT)
+	i(GLUT_ACTION_GLUTMAINLOOP_RETURNS)
+	i(GLUT_ACTION_CONTINUE_EXECUTION)
+	i(GLUT_ACTION_ON_WINDOW_CLOSE)
 	}
 	else
 #endif /* def GTK_API_VERSION */
