@@ -1,5 +1,10 @@
 #include <stdio.h>
-#include "../include/GL/glut.h"
+
+#if defined(HAVE_FREEGLUT)
+#include "../include/GL/freeglut.h"
+#else
+#include <GL/glut.h>
+#endif
 
 #define PROGRAM "glversion"
 
@@ -10,21 +15,22 @@ int main(int argc, char **argv)
   char *renderer = NULL;
   char *extensions = NULL;
   GLuint idWindow = 0;
+  int	glutVersion;
 
   glutInit(&argc, argv);
   glutInitWindowSize(1,1);
-  //glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH | GLUT_ALPHA);
   glutInitDisplayMode(GLUT_RGBA);
   idWindow = glutCreateWindow(PROGRAM);
   glutHideWindow();
 
+  glutVersion = glutGet(0x01FC);
   version =     (char*)glGetString(GL_VERSION);
   vendor =      (char*)glGetString(GL_VENDOR);
   renderer =    (char*)glGetString(GL_RENDERER);
   extensions =  (char*)glGetString(GL_EXTENSIONS);
 
-  printf("VERSION=%s\nVENDOR=%s\nRENDERER=%s\nEXTENSIONS=%s\n",
-    version,vendor,renderer,extensions);
+  printf("GLUT=%d\nVERSION=%s\nVENDOR=%s\nRENDERER=%s\nEXTENSIONS=%s\n",
+    glutVersion,version,vendor,renderer,extensions);
 
   glutDestroyWindow(idWindow);
   return(0);
